/**
 * @author DOGO
 * @module WebViewer3D
 * @copyright 2017 Esko Software BVBA. All rights reserved.
 */

define([
    "underscore",
    "backbone",
    "three",
    "./LogMessage"
], function (
    _,
    bb,
    THREE,
    LogMessage
)
{

    "use strict";

    /**
     * @alias module:WebViewer3D
     * @class WV3DRendererCache
     * This class is a container for all the data and rendering options related to a particular scene
     */
    return bb.Model.extend({

        initialize: function ()
        {
            LogMessage.Log("Creating WV3DRendererCache Object");

            this.Reset();

            this.set("_polygon_offset_factor", 0.5);
            this.set("_polygon_offset_units", 1);    // For unknown reasons, the units factor is scaled down somewhere by a huge factor.
                                                     // I had to test numbers in the 10000-range to see visual differences, unless zoomed in!
                                                     // Does Three.JS use the OpenGl 1.0 polygon offset (with bias) instead of 1.1 (with units)?

            this.set("_jsonRenderingQuality","Designer");//can be "Designer" or "Visualizer". This is the quality indicated in the JSON file.
            this.set("_queriedRenderingQuality","JSON");//can be "Designer", "Visualizer", or "JSON". This is the quality that the user wants to switch to
                                                        // that will be reflected in _RenderingQuality upon next scene load or material update. If JSON,
                                                        //then _RenderingQuality will be set to what is contained in _jsonRenderingQuality.

            this.set("_visualizerTexturesPath","lighting_environments/visualizer_quality/");

            this.set("_initialCameras",
                {
                    "defaultcamera"://default camera in use
                        {
                            "cam_type": "PERSPECTIVE",
                            "azimuthAngle": 0,
                            "elevationAngle": 0,
                            "fieldOfView": 30,
                            "scale": 1,
                            "panOffset": {"x": 0, "y": 0}
                        }
                });

            this.set("cameras", $.extend(true, {}, this.get("_initialCameras")));//deep copy of default camera asset;
        },

        UpdateJSONRenderingQuality: function(__jsonRenderingQuality)
        {
            this.set("_jsonRenderingQuality",__jsonRenderingQuality);

            const queriedRenderingQuality = this.get("_queriedRenderingQuality");
            switch(queriedRenderingQuality)
            {
                case "Designer":
                case "Visualizer":
                    this.set("_RenderingQuality", queriedRenderingQuality);
                    break;
                default://JSON
                    this.set("_RenderingQuality", __jsonRenderingQuality);
            }
        },

        UpdateEnvironmentName: function(__environmentName)
        {
            this.set("_EnvironmentName", __environmentName);
            this.get("textures")["Visualizer"]["path"] = this.get("_visualizerTexturesPath") + __environmentName + "/";
        },

        /**
         *
         */
        Reset: function (__coplanarityEpsilon, __boundaryhardedges, __preserveInteractionState)
        {
            if(!__preserveInteractionState)
            {
                this.set("cameras", $.extend(true, {}, this.get("_initialCameras")));//deep copy of default camera asset;
                this.set("environment_settings",{ "rotation": new THREE.Quaternion() });
            }

            this.set("_RenderingQuality", "Designer");//can be "Designer" or "Visualizer". This contains the quality with which the scene is currently viewed
            this.set("_EnvironmentName", "Office");//Must match a folder name from WebViewer3D/Resources/lighting_environments/visualizer_quality/
            this.set("nodeviewproperties", {});//in use for 3JS nodes URL mapping
            this.set("coplanarityepsilon", (__coplanarityEpsilon !== undefined) ? __coplanarityEpsilon : 0.2);//in use
            this.set("boundaryhardedges", (__boundaryhardedges !== undefined) ? __boundaryhardedges : true);
            this.set("geometries", []);//in use for three JS geometries
            this.set("materials",//in use for three JS material (colored faces)
                {
                    "enablehighlights": false,
                    "enablehorizonhighlights": true,
                    "enablehardedgehighlights": true,
                    "enableirradiancemap": true,
                    "enableenvmap": true,
                    "defaultcolor": [30, 30, 30],
                    "default_material": null,
                    "materiallist": {},//The map containing all materials for imported materials
                    "shaders": {}
                });
            this.set("labels",//label assets loaded from JSON
                {
                    "visible": true,
                    "list": {}
                });
            this.set("textures",//in use for three JS textures
                {
                    //Textures used in Designer Quality
                    "Designer":
                        {
                            "path": "lighting_environments/designer_quality/",
                            "irradiancemap":
                                {
                                    "files": ["irr_map_right.PNG",//+X
                                        "irr_map_left.PNG",//-X
                                        "irr_map_top.PNG",//+Y
                                        "irr_map_bottom.PNG",//-Y
                                        "irr_map_front.PNG",//+Z
                                        "irr_map_back.PNG"//-Z
                                    ],
                                    "data": null,
                                    "loaded": false
                                },
                            "envmap":
                                {
                                    "glossy":
                                        {
                                            "files": ["env_map_glossy_right.PNG",//+X
                                                "env_map_glossy_left.PNG",//-X
                                                "env_map_glossy_top.PNG",//+Y
                                                "env_map_glossy_bottom.PNG",//-Y
                                                "env_map_glossy_front.PNG",//+Z
                                                "env_map_glossy_back.PNG"//-Z
                                            ],
                                            "data": null,
                                            "loaded": false
                                        },
                                    "matte":
                                        {
                                            "files": ["env_map_mate_right.PNG",//+X
                                                "env_map_mate_left.PNG",//-X
                                                "env_map_mate_top.PNG",//+Y
                                                "env_map_mate_bottom.PNG",//-Y
                                                "env_map_mate_front.PNG",//+Z
                                                "env_map_mate_back.PNG"//-Z
                                            ],
                                            "data": null,
                                            "loaded": false
                                        }
                                },
                            "backdrop":
                                {
                                    "loaded": false,
                                    "data": null,
                                    "original_width": null,
                                    "original_height": null
                                }
                        },
                    //Textures used in Visualizer Quality
                    "Visualizer":
                        {
                            "path": this.get("_visualizerTexturesPath") + "Office/",//Initial default path
                            "irradiancemap":
                                {
                                    "files": ["cubic_irradiance_xpos.jpg",//+X
                                        "cubic_irradiance_xneg.jpg",//-X
                                        "cubic_irradiance_ypos.jpg",//+Y
                                        "cubic_irradiance_yneg.jpg",//-Y
                                        "cubic_irradiance_zpos.jpg",//+Z
                                        "cubic_irradiance_zneg.jpg"//-Z
                                    ],
                                    "data": null,
                                    "loaded": false
                                },
                            "envmap":
                                {
                                    "files": ["cubic_blur_00_xpos.jpg",//+X
                                        "cubic_blur_00_xneg.jpg",//-X
                                        "cubic_blur_00_ypos.jpg",//+Y
                                        "cubic_blur_00_yneg.jpg",//-Y
                                        "cubic_blur_00_zpos.jpg",//+Z
                                        "cubic_blur_00_zneg.jpg",//-Z
                                        "cubic_blur_02_xpos.jpg",//+X
                                        "cubic_blur_02_xneg.jpg",//-X
                                        "cubic_blur_02_ypos.jpg",//+Y
                                        "cubic_blur_02_yneg.jpg",//-Y
                                        "cubic_blur_02_zpos.jpg",//+Z
                                        "cubic_blur_02_zneg.jpg",//-Z
                                        "cubic_blur_04_xpos.jpg",//+X
                                        "cubic_blur_04_xneg.jpg",//-X
                                        "cubic_blur_04_ypos.jpg",//+Y
                                        "cubic_blur_04_yneg.jpg",//-Y
                                        "cubic_blur_04_zpos.jpg",//+Z
                                        "cubic_blur_04_zneg.jpg",//-Z
                                        "cubic_blur_06_xpos.jpg",//+X
                                        "cubic_blur_06_xneg.jpg",//-X
                                        "cubic_blur_06_ypos.jpg",//+Y
                                        "cubic_blur_06_yneg.jpg",//-Y
                                        "cubic_blur_06_zpos.jpg",//+Z
                                        "cubic_blur_06_zneg.jpg",//-Z
                                        "cubic_blur_08_xpos.jpg",//+X
                                        "cubic_blur_08_xneg.jpg",//-X
                                        "cubic_blur_08_ypos.jpg",//+Y
                                        "cubic_blur_08_yneg.jpg",//-Y
                                        "cubic_blur_08_zpos.jpg",//+Z
                                        "cubic_blur_08_zneg.jpg"//-Z
                                    ],
                                    "data": null,
                                    "loaded": false
                                },
                            "spectrum":
                                {
                                    "file": "spectrum_00.png",
                                    "data": null,
                                    "loaded": false,
                                    "queried": false
                                }
                        },
                    "backdrop":
                        {
                            "loaded": false,
                            "data": null,
                            "original_width": null,
                            "original_height": null
                        },
                    //Texture loaded from Json
                    "FromFile": {}
                });
            this.set("view_settings",
                {
                    "mode": 1,//0: floating mode (not implemented), 1: azim_elev mode (default)
                    "model_xform": new THREE.Matrix4()
                });
            this.set("camera_preset",
                {
                    "eye_position": null
                });
            this.set("lights", {});//in use for ambiant and directional lights
            this.set("reflections",
                {
                    "enabled": false,//reflects what is in json structure
                    "opacity": 0.25,
                    "fade": false,
                    "fade_size": 1.2,  // Optional.
                    "fade_position": 0.95  //Optional.
                });
        }


    });

});
