import { SelectionModel } from "./TextSelectionLayerModel";
import _ from "underscore";

const DEFAULT_FONT_SIZE = 4;
const isOkFontSize = (font, size) => font && font.fontSize < size;

/**
 * This model is used with the Check Font Sizes tool to display/highlight text that has a smaller font size than a set value
 * It can be used in a TextSelectionLayerModel (ie. it has the getSelection, getHighlight and getContent methods)
 */
export default class extends SelectionModel {
	constructor(f, channelName, textModel) {
		super(f, channelName);
		this._textSelectionModel = textModel;

		/**
		 * The displayed selection. It is saved as a member rather than recomputing every time as it is quite expensive to compute
		 * @member {Array.<TextSelectionModel.Character>}
		 */
		this._selection = [];

		/**
		 * the current highlighted text box. -1 means that none is highlighted
		 */
		this._i = -1;

		/**
		 * The max font size of all characters in the text
		 */
		this._max = _.last(textModel.getFontSizes()) || 1;

		/**
		 * Whenever the text changes recompute
		 */
		textModel.on("hasText.changed", () => {
			this._max = (_.last(textModel.getFontSizes()) || 1) + 0.1; // do + 0.1 because otherwise the characters with max font size will never be highlighted
			this._emit("changed.max");
			this._calculateSelection("value");
		});

		/*
		 * a function that throttles the calculation of selection.
		 */
		this._calculateSelection = (() => {
			var emits = []; // these are all change events that will be emitted when a calculation is done.

			var doIt = _.throttle(() => {
				// the actual calculation of the selection is throttled
				// so that when calling calculateSelection multiple times in the same event loop it will only be calculated once

				var highlights = this.getHighlight();

				this._selection = this._textSelectionModel.getCharacters().map(c => {
					var cProxy = Object.create(c); // proxy of the character so that we don't override the original
					if (isOkFontSize(cProxy, this.getValue())) {
						// font is smaller  than threshold
						if (_.find(highlights, h => h.elementIndex === c.elementIndex)) {
							// if the character is inside the highlighted text box we give it a special color
							cProxy.color = "rgba(226, 152, 29, 0.7)";
						} else {
							// otherwise just red
							cProxy.color = "rgba(222, 45, 23, 0.6)";
						}
						cProxy.lineIndex = c.lineIndex + "-highlight"; // lineIndex is changed to make sure that it is seen as a different box in TextSelectionLayerModel
					} else {
						cProxy.color = "rgba(240, 240, 240, 0.5)"; // bigger fonts are also added but get a light grey color
					}
					return cProxy;
				});
				_.each(_.uniq(emits), emit => this._emit("changed." + emit)); // emit all events
				emits = [];
			}, 16);

			return (...emit) => {
				_.each(emit, emit => {
					emits.push(emit); // add the event that should be emitted after calculation
				});
				doIt();
			};
		})();
	}
	/**
	 * Get the maximum font size of all characters in the text
	 * @returns {Number}
	 */
	getMax() {
		return this._max;
	}
	/**
	 * Sets the font threshold
	 * @param {Number} value - the threshold to use
	 */
	setValue(value) {
		this.f.setOptionCallback("PREFERENCES_MINIMUM_FONT_SIZE", value, value => {
			this._showHighlights(false);
			this._i = -1;
			this._calculateSelection("value");
		});
	}
	/**
	 * returns the current threshold
	 */
	getValue() {
		var value = this.f.getOption("PREFERENCES_MINIMUM_FONT_SIZE") || DEFAULT_FONT_SIZE;
		value = Math.max(0, Math.min(Math.round(value * 10) / 10, this._max));
		return value;
	}

	/**
	 * Get the selection to be showed in TextSelectionLayerModel. Not the same as getHighlights.
	 * getSelection also includes characters that are bigger than the font size but get another color
	 */
	getSelection() {
		return this._selection;
	}
	/**
	 * for what page is this used?
	 */
	getPage() {
		return this._textSelectionModel.getPage();
	}

	/**
	 * Only used internally! sets if we want to use the highlightIndex highlight method or the startpoint/stoppoint highlight method
	 * @param {Boolean} show
	 */
	_showHighlights(show) {
		if (this._IsShowHighlights !== show) {
			this._IsShowHighlights = show;
			this._calculateSelection("highlight");
		}
	}
	/**
	 * All characters that are smaller than the font size.
	 */
	getHighlights() {
		return _.chain(this._textSelectionModel.getCharacters())
			.filter(c => isOkFontSize(c, this.getValue()))
			.groupBy("lineIndex")
			.values()
			.value();
	}
	/**
	 * get content is used for the annotation content used by TextSelectionLayerModel
	 */
	getContent() {
		return this.f.t("FontSizeTooSmall", this.getValue());
	}
	/**
	 * getHighlight is used by TextSelectionLayerModel to know for what it has to create an annotation
	 * @returns {Array.<TextSelectionModel.Character>}
	 */
	getHighlight() {
		var highlights = [];
		if (this._startHighlight && this._endHighlight) {
			// if we have start and end highlihgt points get all characters in that rect
			highlights = _.filter(this._textSelectionModel.getInRect(this._startHighlight, this._endHighlight), c =>
				isOkFontSize(c, this.getValue())
			);
		} else if (this._IsShowHighlights) {
			highlights = this.getHighlights()[this.getHighlightedIndex()];
		}
		return highlights;
	}
	getHighlightedIndex() {
		return this._i;
	}
	hitCount() {
		return this.getHighlights().length;
	}
	/**
	 * used by next and previous buttons to step through the highlights
	 * @param {Number} dx
	 */
	_d(dx) {
		this._startHighlight = null;
		this._endHighlight = null;
		const hitCount = this.hitCount();
		if (hitCount > 0) {
			this._showHighlights(true);
			const hitCount = this.hitCount();
			this._i = (this._i + dx + hitCount) % this.hitCount();
			this._calculateSelection("highlight", "highlightIndex");
		}
	}
	next() {
		this._d(1);
	}
	previous() {
		this._i = Math.max(this._i, 0); // -1 means no highlighted. doing -1 when i == -1 we want to go to the last one
		this._d(-1);
	}
	setStartHighlight(startPoint) {
		this._startHighlight = startPoint;
		this._showHighlights(false);
		this._calculateSelection("highlight");
	}
	getStartHighlight() {
		return this._startHighlight;
	}
	getEndHighlight() {
		return this._endHighlight;
	}
	setEndHighlight(endPoint) {
		this._endHighlight = endPoint;
		this._showHighlights(false);
		this._calculateSelection("highlight");
	}
}
