import _ from "underscore";
import AbstractTool from "./AbstractTextTool";
import FontSelectionLayerModel from "./FontSelectionLayerModel";
import ListenerUtil from "@esko/web-viewer/utils/ListenerUtil";

var DEFAULTSTATE = "default";
var SELECTEDSTATE = "selected";

/**
 * @exports FontSelectionTool
 * @memberof SvgModule
 * @class The FontSelectionTool allows to check which font a certain character has
 * @constructor
 * @augments {Core.AbstractTool}
 *
 * @param {Core.ViewerFacade} f - Viewer facade
 * @param {SvgModule.TextSelectionModel}
 */
var FontSelectionTool = function(f, name, textSelectionModel) {
	AbstractTool.call(this, f, name, textSelectionModel);
	ListenerUtil.mixin(this, f);

	/**
	 * @member {FontSelectionLayerModel}
	 * @private
	 */
	this._fontSelectionLayerModel = new FontSelectionLayerModel(f, this.getIID(), textSelectionModel, this.sceneToWorldSize.bind(this));
	this._fontSelectionLayerModel.on("popoverRemoved", _.debounce(() => this.changeState(DEFAULTSTATE), 16));

	textSelectionModel.on("hasText.changed", hasText => {
		if (this.isActive() && !hasText) {
			this.deactivate();
		}
	});

	/**
	 * DEFAULT STATE -> nothing set
	 */
	this.registerState(DEFAULTSTATE, {
		enterState: () => {
			this._fontSelectionLayerModel.setPoint(null);
			this._fontSelectionLayerModel.setHoverPoint(null);
			this._fontSelectionLayerModel.hidePopover();
		},

		click: e => {
			var curLoc = this.getWorldPoint(e);
			this._fontSelectionLayerModel.setPoint(curLoc);
			this._fontSelectionLayerModel.showPopover();
			this.changeState(SELECTEDSTATE);
		},

		mousemove: e => {
			var curLoc = this.getTextPoint(e);
			var c = this.getCloseCharacter(curLoc);
			this._fontSelectionLayerModel.setHoverPoint(this.getWorldPoint(e));
			if (c) {
				this.setCursor("rulerCursor");
			} else {
				this.setCursor(this.getDefaultCursor());
			}
		},

		/**
		 * no context menu!
		 */
		contextmenu: _.bind(function(e) {
			e.preventDefault();
			e.stopPropagation();
		}, this),

		canBeInterrupted: false
	});

	/**
	 * SELECTEDSTATE
	 */
	this.registerState(
		SELECTEDSTATE,
		_.defaults(
			{
				enterState: () => {},
				click: e => {
					var curLoc = this.getWorldPoint(e);
					var c = this.getCloseCharacter(curLoc);
					if (c) {
						this._fontSelectionLayerModel.setPoint(curLoc);
						this._fontSelectionLayerModel.showPopover();
					} else {
						this.changeState(DEFAULTSTATE);
					}
				},
				canBeInterrupted: false
			},
			this.getState(DEFAULTSTATE)
		)
	);

	this.registerActiveEvents(["click", "mousemove", "contextmenu"]);

	/**
	 * @override
	 */
	this._override("initialize", function(widget, toolManager) {
		this._super(widget, toolManager);
		this.changeState(DEFAULTSTATE);
	});

	/**
	 * @override
	 */
	this._override("activate", function(interruptRequest) {
		this._super(interruptRequest);
		this.f.getModule("viewport").addLayer(this.getIID(), this._fontSelectionLayerModel);
	});

	this._override("deactivate", () => {
		this._super();
		this._fontSelectionLayerModel.hidePopover();
		this.f.getModule("viewport").removeLayer(this.getIID());
	});

	// activate text selection tool
	this.passiveKeyEvents.push([
		"f",
		e => {
			if (textSelectionModel.hasText()) {
				e.preventDefault();
				f.emit("toolManager.activateTool", this);
			}
		},
		"keypress"
	]);
};

// extend from the AbstractTool
// inherit
FontSelectionTool.prototype = Object.create(AbstractTool.prototype);
FontSelectionTool.prototype.constructor = FontSelectionTool;

export default FontSelectionTool;
