import ViewportLayerModel from "@esko/web-viewer/viewport/model/ViewportLayerModel";
import TextSelectionPopoverModel from "./TextSelectionPopoverModel";
import ListenerUtil from "@esko/web-viewer/utils/ListenerUtil";
import LineworkWrapper from "@esko/web-viewer/utils/LineworkWrapper";
import _ from "underscore";
import DrawUtils from "@esko/web-viewer/utils/DrawUtils";
import { transform } from "./TextTransformation";
import { rectFromSelection } from "./TextUtils";
import AbstractModel from "../core/AbstractModel";

/**
 * Class that is used to show selections in the TextSelectionLayerModel
 * The TextSelectionLayerModel listens to "changed.*" to rerender its linework. The linework is created by calling
 * getSelection() on this selection model.
 */
class SelectionModel extends AbstractModel {
	/**
	 * should return a list of characters that will be 'selected'
	 * @returns {Array.<TextSelectionModel.Character>}
	 */
	getSelection() {
		throw new Error("Not Implemented");
	}
	/**
	 * returns a list of characters. When a user clicks the 'make annotation' button
	 * the charachters returned by this method is used to create a bounding rect
	 * @returns {Array.<TextSelectionModel.Character>}
	 */
	getHighlight() {
		throw new Error("Not Implemented");
	}
	/**
	 * @returns {String} Content of the annotation
	 */
	getContent() {
		throw new Error("Not Implemented");
	}
}

/**
 * @exports TextSelectionLayerModel
 * @memberof SvgModule
 * @augments ViewportLayerModel
 * @class This class gives the linework to render selected text and show popover.
 * @constructor
 *
 * @param {Core.ViewerFacade} f - Viewer facade
 * @param {String} ID - the ID of the layer model
 * @param {TextSelectionModel} textSelectionModel - Model of the text selection tool
 * @param {AbstractTool} tool - tool
 * @param {SelectionModel} selectionModel - An abstract model that should have following methods:
 * getSelection: all characters that will be displayed as selected
 * getHighlight: this call is used to get the characters used for an annotation,
 * getContent: the text used in the annotation
 */
var TextSelectionLayerModel = function(f, ID, textSelectionModel, tool, selectionModel) {
	ViewportLayerModel.call(this, f, ID);
	ListenerUtil.mixin(this, f);

	var updateSelection = () => {
		this.hidePopover();
		this._selection = selectionModel.getSelection();
		this._calculateTransformation();
	};

	// Variables
	this._selection = [];
	this._textSelectionModel = textSelectionModel;
	textSelectionModel.on("hasText.changed", updateSelection);
	this._selectionModel = selectionModel;

	tool.on("activated", updateSelection);
	selectionModel.on("changed.*", updateSelection);

	this._registerListener("parameters.changed", this._calculateTransformation, this, this.f.getDocumentController());
	this._registerListener("transform.viewportWidget.transformChanged", this._calculateTransformation, this);

	this._textSelectionPopover = null;
	this._textSelectionPopoverListener = null;
};

// inherit
TextSelectionLayerModel.prototype = Object.create(ViewportLayerModel.prototype);
TextSelectionLayerModel.prototype.constructor = TextSelectionLayerModel;

/**
 * Calculates the transformation from document coordinates to world coordinates
 */
TextSelectionLayerModel.prototype._calculateTransformation = function() {
	var page = this._textSelectionModel.getPage();
	if (page !== null && page !== undefined) {
		this.transform = transform(page);
	}
	this.modelChanged();
};

TextSelectionLayerModel.prototype.modelChanged = _.throttle(ViewportLayerModel.prototype.modelChanged, 17);
/**
 * Start showing the textSelection popover
 */
TextSelectionLayerModel.prototype.showPopover = function(location, popoverOptions) {
	if (!this._textSelectionPopover) {
		this._textSelectionPopover = new TextSelectionPopoverModel(this.f, "TextSelectionPopover", this._selectionModel);
		this._textSelectionPopoverListener = this._registerListener(
			"popupClosed",
			function(e) {
				this.hidePopover();
			},
			this,
			this._textSelectionPopover
		);
		this.addPopoverModel(this._textSelectionPopover);
	}
	this._textSelectionPopover.setPopoverOptions(popoverOptions || {});
	this._textSelectionPopover.setTextContent(this._selectionModel.getContent());
	var worldTextFrames = this._createLinework(this._selectionModel.getHighlight()).map(lw => lw.rect);
	this._textSelectionPopover.setWorldTextFrames(worldTextFrames);
	this._textSelectionPopover.setPopoverLocation(location);
	this.modelChanged();
};

TextSelectionLayerModel.prototype.hasLinework = function() {
	return true;
};

TextSelectionLayerModel.prototype.getLinework = function() {
	return this._createLinework(this._selection);
};

/**
 * creates lineworkwrappers for displaying selection rects.
 * @param {Array.<TextSelectionModel.Character>} selection
 * @returns {Array.<LineworkWrapper>}
 */
TextSelectionLayerModel.prototype._createLinework = function(selection) {
	return _.chain(_.groupBy(selection, "lineIndex")) // characters with the same lineIndex are on the same line and can use the same linework wrapper
		.values()
		.map(line => _.values(_.groupBy(line, "color"))) // a single line (same lineIndex) can contain different colors, those will be different linework rects
		.flatten(true)
		.map(line => {
			// here we have all characters in selection with the same line index and color. But it could be that some characters are not selected (eg. when using text search)
			// We would like to break up line into parts that follow each other;
			var currentPart = [line[0]];
			var parts = [currentPart];
			line.slice(1).forEach(c => {
				var prevCharacter = currentPart[currentPart.length - 1];
				if (c.elementIndex - prevCharacter.elementIndex === 1) {
					// if the elementIndex directly follows the previous elementIndex they will be one linework rect, so add it to the currentpart
					currentPart.push(c);
				} else {
					// start another linework rect
					currentPart = [c];
					parts.push(currentPart);
				}
			});
			return parts;
		})
		.flatten(true)
		.map((
			parts // Here we finally have all characters that become one linework wrapper, take the color and the bounding rect
		) => ({
			color: parts[0].color || "rgba(51, 153, 255, 0.7)",
			rect: rectFromSelection(parts)
		}))
		.map(({ rect, color }) => {
			var lww = new LineworkWrapper();
			lww.rect = rect;
			lww.fillcolor = color;
			lww.type = DrawUtils.LWTYPE_RECT;
			return lww;
		})
		.value();
};

/**
 * Hide the textSelection popover
 */
TextSelectionLayerModel.prototype.hidePopover = function() {
	if (this._textSelectionPopover) {
		this.removePopoverModel(this._textSelectionPopover.getID());
		this.removePopover(this._textSelectionPopover.getID());
		this._unregisterListener(this._textSelectionPopoverListener);
		this._textSelectionPopover = null;
	}
};

/**
 * Override the model changed, so the popovermodel is updated when needed.
 * @override
 */

/**
 * Get Popover Model
 * @returns Popover Model
 */
TextSelectionLayerModel.prototype.getPopoverModel = function() {
	return this._textSelectionPopover;
};

export default TextSelectionLayerModel;
export { SelectionModel };
