import ViewerToolbarButton from "@esko/web-viewer/layout/component/ViewerToolbarButton";
import ToolbarComponent from "@esko/toolbar/ToolbarComponent";
import Slider from "@esko/slider/Slider";
import Tooltip from "@esko/tooltip";
import $ from "jquery";
import { rectFromSelection } from "./TextUtils";

/**
 * Button used for the Check Font Size tool.
 * Also specifies the subtoolbar for the button.
 * @param {ViewerFacade} f
 * @param {checkFontSizeModel} checkFontSizeModel
 * @param {AbstractTool} tool
 */
export default class extends ViewerToolbarButton {
	constructor(f, checkFontSizeModel, tool) {
		super(f, "HighlightText", "icon-WCR_Viewer-Check-Font-Size", f.isMobile());
		this.listenToTool(tool, true);
		this.setTooltip("viewer", "CheckFontSizesTT");

		var subToolbar = new ToolbarComponent("HighlightTextFontThreshold");

		/**
		 * input field that doubles as Label for the font size threshold.
		 */
		var label = $("<input size='3' step='0.1' min='0' class='viewerApp-font-threshold-input' type='number'>")
			.val(0)
			.on("input", () => label.val() && checkFontSizeModel.setValue(label.val()))
			.on("keypress", e => {
				if (e.key === "Enter") {
					checkFontSizeModel.next();
				}
			});

		/**
		 * Slider to change the font size threshold
		 */
		var slider = new Slider({
			max: 1,
			on: {
				"value.change": value => checkFontSizeModel.setValue(value)
			}
		});
		slider.element.addClass("viewerApp-font-threshold-slider");

		/**
		 * some classes used in the subtoolbar defined as consts
		 */
		const TEXTSEARCH = "viewerApp-text-search";
		const HITCOUNT = `${TEXTSEARCH}-hitCount`;
		const NOHITS = `${HITCOUNT}-none`;

		const STEPTHROUGH = `${TEXTSEARCH}-stepThrough`;
		const STEPTHROUGH_TOGGLE = `${STEPTHROUGH}-toggle`;
		const STEPTHROUGH_NEXT = `${STEPTHROUGH}-next`;
		const STEPTHROUGH_PREVIOUS = `${STEPTHROUGH}-previous`;

		// the 'Step through' elements
		var highlight = $(`<span class="${HITCOUNT}"></span>
		<div class="${STEPTHROUGH}">
			<span class="btn btn-xs ${STEPTHROUGH_NEXT} ${STEPTHROUGH_TOGGLE}"><i class="icon icon-sm icon-WCR_Chevron-Down12" /></span>
			<span class="btn btn-xs ${STEPTHROUGH_PREVIOUS} ${STEPTHROUGH_TOGGLE}"><i class="icon icon-sm icon-WCR_Chevron-Up12" /></span>
		</div>`);

		subToolbar.element
			.addClass("viewerApp-text-search-toolbar")
			.append($("<span>").text(f.t("fontThreshold") + ": "))
			.append(slider.element)
			.append(label)
			.append($("<span>", { text: "pt" }))
			.append(highlight);
		this.addSubtoolbarComponent(subToolbar);

		// add tooltips
		f.loadBundle("viewer").then(() => {
			[STEPTHROUGH_NEXT, STEPTHROUGH_PREVIOUS].forEach(c => {
				// use c as tooltip key
				new Tooltip(null, subToolbar.element.find("." + c), f.getContext(), f.getContext(), f.isMobile(), f.t(c));
			});
		});

		// Update the highlight part of the subtoolbar
		var updateHighlight = () => {
			const hitCount = checkFontSizeModel.hitCount();
			const $hitCount = subToolbar.element.find(`.${HITCOUNT}`);
			$hitCount.text(`${checkFontSizeModel.getHighlightedIndex() + 1}/${hitCount}`);
			$hitCount.toggleClass(NOHITS, checkFontSizeModel.hitCount() === 0);
			highlight.find(`.${STEPTHROUGH_TOGGLE}`).toggleClass("disabled", hitCount <= 1);
		};

		// On change of the threshold
		var onValueChange = () => {
			var val = checkFontSizeModel.getValue();
			label.val(val);
			slider.setValue(val);
			updateHighlight();
		};

		/**
		 * Zoom to the highlighted hit
		 */
		var zoomToHighlight = () => {
			var hit = checkFontSizeModel.getHighlight();
			if (hit) {
				f.emit("viewport.fitInRect", {
					rect: rectFromSelection(hit),
					animated: true,
					constrain: true,
					maxZoomLevel: 0
				});
			}
		};

		// On highlightIndex change, zoom and update the highlight part of the subtoolbar
		checkFontSizeModel.on("changed.highlightIndex", () => {
			zoomToHighlight();
			updateHighlight();
		});

		/**
		 * OnMaxChange
		 */
		var onMaxChange = () => {
			var max = checkFontSizeModel.getMax();
			slider.setOptions({ max: max });
			label.attr("max", max);
			var i = 2;
			var maxTemp = max;
			while (maxTemp > 1) {
				i++;
				maxTemp = maxTemp / 10;
			}
			label.css({ width: i + "em" });
		};

		checkFontSizeModel.on("changed.value", onValueChange);
		onValueChange();

		highlight.on("click", `.${STEPTHROUGH_NEXT}`, () => {
			checkFontSizeModel.next();
		});

		highlight.on("click", `.${STEPTHROUGH_PREVIOUS}`, () => {
			checkFontSizeModel.previous();
		});

		checkFontSizeModel.on("changed.max", onMaxChange);
		onMaxChange();
	}
}
