import TextSelectionLayerModel from "@esko/web-viewer/svg/TextSelectionLayerModel";

import _ from "underscore";
import AbstractTool from "./AbstractTextTool";
import CoreConstants from "@esko/web-viewer/core/CoreConstants";
import FontSelectionLayerModel from "./FontSelectionLayerModel";
import ListenerUtil from "@esko/web-viewer/utils/ListenerUtil";

var DEFAULTSTATE = "default";
var DRAGGINGSTATE = "dragging";

/**
 * @exports FontSelectionTool
 * @memberof SvgModule
 * @class The FontSelectionTool allows to select text in SVG files
 * @constructor
 * @augments {Core.AbstractTool}
 *
 * @param {Core.ViewerFacade} f - Viewer facade
 * @param {SvgModule.TextSelectionModel}
 */
var FontSelectionTool = function(f, name, textSelectionModel, checkFontSizesModel) {
	AbstractTool.call(this, f, name, textSelectionModel);
	ListenerUtil.mixin(this, f);

	/**
	 * @member {FontSelectionLayerModel}
	 * @private
	 */
	this._fontSelectionLayerModel = new FontSelectionLayerModel(
		f,
		this.getIID(),
		textSelectionModel,
		this.sceneToWorldSize.bind(this),
		checkFontSizesModel
	);
	var textSelectionLayerModel = new TextSelectionLayerModel(f, "checkfontsizeslayer", textSelectionModel, this, checkFontSizesModel);

	checkFontSizesModel.on("changed.highlightIndex", () => this.changeState(DEFAULTSTATE));
	textSelectionModel.on("hasText.changed", hasText => {
		if (this.isActive() && !hasText) {
			this.deactivate();
		}
	});

	// The events that are registered
	const events = ["click", "mousedown", "mouseup", "mousemove", "contextmenu"];

	// an object that has a noop function on every event, will be used to extend from when defining states
	const noop = function() {};
	const noops = _.reduce(
		events,
		(mem, event) => {
			mem[event] = noop;
			return mem;
		},
		{}
	);

	/**
	 * DEFAULT STATE -> click on a text box to select
	 */
	this.registerState(
		DEFAULTSTATE,
		_.defaults(
			{
				enterState: () => {
					this._fontSelectionLayerModel.setPoint(null);
					this._fontSelectionLayerModel.setHoverPoint(null);
				},

				click: e => {
					var curLoc = this.getWorldPoint(e);
					checkFontSizesModel.setEndHighlight(null);
					checkFontSizesModel.setStartHighlight(null);
					this._fontSelectionLayerModel.setPoint(curLoc);
					this._fontSelectionLayerModel.showPopover();
				},
				mousedown: e => {
					if (e.button === 0) {
						var curLoc = this.getTextPoint(e);
						var c = this.getCloseCharacter(curLoc);
						if (!c) {
							// When we do not have a character close by on mouse down, we'll drag a rectangle
							checkFontSizesModel.setEndHighlight(null);
							checkFontSizesModel.setStartHighlight(curLoc);
							this.changeState(DRAGGINGSTATE);
						}
					}
				},
				mousemove: e => {
					var curLoc = this.getTextPoint(e);
					var c = this.getCloseCharacter(curLoc);
					this._fontSelectionLayerModel.setHoverPoint(this.getWorldPoint(e));
					if (c) {
						this.setCursor("rulerCursor");
					} else {
						this.setCursor(this.getDefaultCursor());
					}
				},
				/**
				 * show popover on rightclick
				 */
				contextmenu: _.bind(function(e) {
					textSelectionLayerModel.showPopover(this.getTextPoint(e), { showCopy: false });
					e.preventDefault();
					e.stopPropagation();
				}, this),

				canBeInterrupted: false
			},
			noops
		)
	);

	// dragging state -> dragging a rectangle to highlight the contents of it
	this.registerState(
		DRAGGINGSTATE,
		_.defaults(
			{
				enterState: e => {
					this._fontSelectionLayerModel.setPoint(null);
					checkFontSizesModel.setEndHighlight(null);
				},
				mousemove: e => {
					var curLoc = this.getTextPoint(e);
					checkFontSizesModel.setEndHighlight(curLoc);
				},
				mouseup: e => {
					if (checkFontSizesModel.getHighlight().length > 0) {
						textSelectionLayerModel.showPopover(this.getTextPoint(e), { showCopy: false });
					}
				},
				click: e => {
					this.changeState(DEFAULTSTATE);
				}
			},
			noops
		)
	);

	this.registerActiveEvents(events);

	/**
	 * @override
	 */
	this._override("initialize", function(widget, toolManager) {
		this._super(widget, toolManager);
		this.changeState(DEFAULTSTATE);
	});

	/**
	 * @override
	 */
	this._override("activate", function(interruptRequest) {
		this._super(interruptRequest);
		this.f.getModule("viewport").addLayer(CoreConstants.TOOLS.TEXTSELECTION, textSelectionLayerModel);
		this.f.getModule("viewport").addLayer(this.getIID(), this._fontSelectionLayerModel);
	});

	this._override("deactivate", interruptRequest => {
		this._super();
		if (!interruptRequest) {
			this._fontSelectionLayerModel.hidePopover();
			this.f.getModule("viewport").removeLayer(this.getIID());
			this.f.getModule("viewport").removeLayer(CoreConstants.TOOLS.TEXTSELECTION);
		}
	});

	// activate tool on f key press
	this.passiveKeyEvents.push([
		"alt+f",
		e => {
			if (textSelectionModel.hasText()) {
				e.preventDefault();
				f.emit("toolManager.activateTool", this);
			}
		},
		"keypress"
	]);
};

// extend from the AbstractTool
// inherit
FontSelectionTool.prototype = Object.create(AbstractTool.prototype);
FontSelectionTool.prototype.constructor = FontSelectionTool;

export default FontSelectionTool;
