import ViewportLayerModel from "@esko/web-viewer/viewport/model/ViewportLayerModel";
import FontSelectionPopoverModel from "./FontSelectionPopoverModel";
import ListenerUtil from "@esko/web-viewer/utils/ListenerUtil";
import _ from "underscore";
import ShapeUtils from "@esko/shapeutils";
import { transform, inverse } from "./TextTransformation";
import LineworkWrapper from "@esko/web-viewer/utils/LineworkWrapper";
import DrawUtils from "@esko/web-viewer/utils/DrawUtils";
import Ruler from "@esko/web-viewer/measure/Ruler";

/**
 * @exports FontSelectionLayerModel
 * @memberof SvgModule
 * @augments ViewportLayerModel
 * @class This layer is used for font selection. This layer should probably be refactored to augment TextSelectionLayerModel instead
 * @constructor
 *
 * @param {Core.ViewerFacade} f - Viewer facade
 * @param {String} ID - the ID of the layer model
 * @param {TextSelectionModel} textSelectionModel - Model of the text selection tool
 */
var FontSelectionLayerModel = function(f, ID, textSelectionModel, sceneToWorldSize, checkFontSizesModel) {
	ViewportLayerModel.call(this, f, ID);
	ListenerUtil.mixin(this, f);

	// Variables
	this._textSelectionModel = textSelectionModel;
	this._textSelectionModel.on("changed", this._emit.bind(this, "modelChanged"));
	this._fontSelectionPopover = null;
	this._fontSelectionPopoverListener = null;
	this._sceneToWorldSize = sceneToWorldSize;
	if (checkFontSizesModel) {
		this._checkFontSizesModel = checkFontSizesModel;
		this._checkFontSizesModel.on("changed.highlight", () => this.modelChanged());
	}
};

// inherit
FontSelectionLayerModel.prototype = Object.create(ViewportLayerModel.prototype);
FontSelectionLayerModel.prototype.constructor = FontSelectionLayerModel;

/**
 * Start showing the font selection popover
 */
FontSelectionLayerModel.prototype.showPopover = function() {
	if (!this._fontSelectionPopover) {
		this._fontSelectionPopover = new FontSelectionPopoverModel(this.f, "FontSelectionPopover", this);
		this._fontSelectionPopoverListener = this._registerListener(
			"popupClosed",
			function(e) {
				this.hidePopover();
			},
			this,
			this._fontSelectionPopover
		);

		this.addPopoverModel(this._fontSelectionPopover);
	}
};

/**
 * Hide the font selection popover
 */
FontSelectionLayerModel.prototype.hidePopover = function() {
	if (this._fontSelectionPopover) {
		this.removePopoverModel(this._fontSelectionPopover.getID());
		this.removePopover(this._fontSelectionPopover.getID());
		this._unregisterListener(this._fontSelectionPopoverListener);
		this._fontSelectionPopover = null;
	}
};

/**
 * sets the 'selected' point - the textbox underneath this point will always be highlihgted
 * @param {ShapeUtils.Point} point
 */
FontSelectionLayerModel.prototype.setPoint = function(point) {
	this._point = point;
	if (point === null) {
		this.hidePopover();
	}
	this.modelChanged();
};

/**
 * sets the 'hovered' point
 * @param {ShapeUtils.Point} point
 */
FontSelectionLayerModel.prototype.setHoverPoint = function(point) {
	this._hoverPoint = point;
	this.modelChanged();
};

FontSelectionLayerModel.prototype.setDistance = function(d) {
	this._d = d;
	this.modelChanged();
};

FontSelectionLayerModel.prototype.getPoint = function(point) {
	return this._point;
};

FontSelectionLayerModel.prototype.hasLinework = function() {
	return true;
};

FontSelectionLayerModel.prototype.getLinework = function() {
	const hoverFont = this._getFontAtPoint(this._hoverPoint);
	const font = this.getFont();
	const lwws = [];

	if (font) {
		let lww = new LineworkWrapper();

		lww.rect = font.rect;
		lww.fillcolor = "rgba(51, 153, 255, 0.7)";
		lww.type = DrawUtils.LWTYPE_RECT;
		lwws.push(lww);
	}
	if (hoverFont && (!font || font.elementIndex !== hoverFont.elementIndex)) {
		let lww = new LineworkWrapper();

		lww.rect = hoverFont.rect;
		lww.fillcolor = "rgba(51, 153, 255, 0.7)";
		lww.type = DrawUtils.LWTYPE_RECT;
		lwws.push(lww);
	}
	if (this._checkFontSizesModel && this._checkFontSizesModel.getStartHighlight() && this._checkFontSizesModel.getEndHighlight()) {
		var ruler = new Ruler();
		ruler.startPoint = inverse(this._textSelectionModel.getPage()).transformedPoint(this._checkFontSizesModel.getStartHighlight());
		ruler.endPoint = inverse(this._textSelectionModel.getPage()).transformedPoint(this._checkFontSizesModel.getEndHighlight());

		lwws.push(ruler.getSquareLinework());
	}
	return _.flatten(lwws);
};

/**
 * get font information of characthers at a certain point
 * @param {ShapeUtils.Point} point
 */
FontSelectionLayerModel.prototype._getFontAtPoint = function(point) {
	var fonts = this.getEqualFontCharactersAroundPoint(point);
	var font;
	if (fonts.length > 0) {
		font = {
			fontSize: fonts[0].fontSize,
			fontFamily: fonts[0].fontFamily,
			elementIndex: fonts[0].elementIndex
		};
		font.rect = transform(this._textSelectionModel.getPage()).transformedRect(
			_.reduce(fonts, (mem, val) => mem.calculateArea(val.rect), fonts[0].rect)
		); // reduce all different char rects to a single rect
		font.text = _.pluck(fonts, "character").join("");
	}
	return font;
};

/**
 * returns the font at the location of this._point
 */
FontSelectionLayerModel.prototype.getFont = function() {
	return this._getFontAtPoint(this._point);
};

/**
 * First checks for the character at a certain @param point. If such a point exists. It goes looking at all characters with the same lineIndex
 * All characters next to that one characther with the same fontSize and same fontFamily are returned.
 * @param {ShapeUtils.Point} point in world coordinates
 * @returns {Array.<TextSelectionModel.Character>}
 */
FontSelectionLayerModel.prototype.getEqualFontCharactersAroundPoint = function(point) {
	if (!point) {
		return [];
	}
	var d = this._sceneToWorldSize(new ShapeUtils.Size(18, 18)).width;
	const char = this._textSelectionModel.getCharacter(inverse(this._textSelectionModel.getPage()).transformedPoint(point), d);
	if (char) {
		// Partition the characters with the same line index in characters 'before' and 'after'.
		let [beforeChars, afterChars] = _.chain(this._textSelectionModel.getCharacters())
			.filter(c => c.lineIndex === char.lineIndex)
			.partition(x => x.elementIndex <= char.elementIndex)
			.value();

		// takes the head of an array as long as the characters have the same font size and family as the characther at point
		const filter = arr => {
			let x = _.findIndex(arr, c => c.fontSize !== char.fontSize && c.fontFamily !== char.fontFamily);
			if (x === -1) {
				x = arr.length;
			} else {
				x = x + 1;
			}
			return _.first(arr, x);
		};
		beforeChars = filter(beforeChars.reverse()).reverse();
		afterChars = filter(afterChars);
		return beforeChars.concat(afterChars);
	} else {
		return [];
	}
};

export default FontSelectionLayerModel;
