/**
 * @namespace SvgModule
 */
import AbstractModule from "@esko/web-viewer/core/AbstractModule";

import CompareConstants from "@esko/web-viewer/compare/CompareConstants";
import SvgLayerModel from "./SvgLayerModel";
import SvgDiffLayerModel from "./SvgDiffLayerModel";
import TextSelectionTool from "./TextSelectionTool";
import TextSelectionModel from "./TextSelectionModel";
import TextSearchModel from "./TextSearchModel";
import LayoutConstants from "@esko/web-viewer/layout/LayoutConstants";
import ToolbarComponent from "@esko/toolbar/ToolbarComponent";
import ButtonGroup from "@esko/toolbar/ButtonGroup";
import ViewerToolbarButton from "@esko/web-viewer/layout/component/ViewerToolbarButton";
import _ from "underscore";
import FontSelectionTool from "@esko/web-viewer/svg/FontSelectionTool";
import TextSearchToolbarComponent from "./TextSearchToolbarComponent";
import CheckFontSizesButton from "@esko/web-viewer/svg/CheckFontSizesButton";
import CoreConstants from "@esko/web-viewer/core/CoreConstants";
import CheckFontSizesTool from "@esko/web-viewer/svg/CheckFontSizesTool";
import CheckFontSizesModel from "./CheckFontSizesModel";

/**
 * Module for enable viewing of svg files
 * @param {ViewerFacade} f
 * @param {Boolean} silenced
 */
var SvgModule = function(f, silenced) {
	AbstractModule.call(this, f, silenced, "svg");

	/**
	 * @member {SvgLayerModel} SvgLayerModel
	 * @private
	 */
	this._layerModel = null;

	/**
	 * @member {SvgDiffLayerModel} SvgDiffLayerModel
	 * @private
	 */
	this._diffLayerModel = null;

	/**
	 * The mother of all text models
	 */
	this._textSelectionModel = new TextSelectionModel(f, [this.getPrivateChannelName(), "textSelection", "model"]);

	/**
	 * Model used for searchin
	 */
	this._textSearchModel = new TextSearchModel(f, [this.getPrivateChannelName(), "textSearch", "model"], this._textSelectionModel);

	/**
	 * Model used for check font sizes tool
	 */
	this._checkFontSizesModel = new CheckFontSizesModel(f, [this.getPrivateChannelName(), "fontSizes", "model"], this._textSelectionModel);

	// Add text tools to the toolbar
	this._textSelectionTool = new TextSelectionTool(f, this._textSelectionModel, this._textSearchModel);
	this._fontSelectionTool = new FontSelectionTool(f, CoreConstants.TOOLS.FONTSELECTION, this._textSelectionModel);
	this._highlightFontTool = new CheckFontSizesTool(
		f,
		CoreConstants.TOOLS.FONTSIZETOOL,
		this._textSelectionModel,
		this._checkFontSizesModel
	);

	f.getDocumentController().whenActiveOrDifferencePageChanged(_.bind(this._setSVGLayerModels, this));
	f.on("doc.*.version.*.page.*.property.svgData.changed", () => this._setSVGLayerModels());

	/**
	 * Which page should be used for text tools?
	 * If no compare mode or show current => use active page
	 * If show difference => use difference page
	 */
	let setPageForText = () => {
		const compare = f.getModule("compare");
		const compareMode = compare && compare.getActiveMode();
		var page;
		if (!compareMode || compareMode.getName() === CompareConstants.mode.SHOW_CURRENT) {
			page = f.getDocumentController().getActivePage();
		} else if (compareMode.getName() === CompareConstants.mode.SHOW_DIFFERENCE) {
			page = f.getDocumentController().getDifferencePage();
		}

		if (page) {
			this._textSelectionModel.setPage(page);
		}
	};

	f.getDocumentController().whenActiveOrDifferencePageChanged(setPageForText);

	this._publishTool(this._textSelectionTool);
	this._publishTool(this._fontSelectionTool);
	this._publishTool(this._highlightFontTool);

	this._textSelectionToolbar = null;

	this._publishToolbarComponent(
		LayoutConstants.ToolbarComponent.TEXTSELECTION,
		_.bind(function(layoutType) {
			var updateTextButtons = _.throttle(
				() => {
					if (this._textSelectionToolbar) {
						const page = this._textSelectionModel.getPage();
						if (page && page.get("textData")) {
							this._textSelectionToolbar.show();
						} else {
							this._textSelectionToolbar.hide();
						}

						const tools = [
							{ tool: this._textSelectionTool, button: textSelectionButton },
							{ tool: this._fontSelectionTool, button: fontSelectionButton },
							{ tool: this._highlightFontTool, button: this._highlightTextButton }
						];

						this.f.emit("toolManager.activeTool", tool => {
							if (tool && _.pluck(tools, "tool").indexOf(tool) > -1 && (!tool.canBeActivated() || !this._textSelectionModel.hasText())) {
								this.f.emit("toolManager.deactivateTool");
							}
						});

						tools.forEach(({ tool, button }) => {
							const canBeActivated = tool.canBeActivated() && this._textSelectionModel.hasText();
							button.setEnabled(canBeActivated);
						});
					}
				},
				20,
				{ leading: false }
			);
			this._textSelectionModel.on("hasText.changed", updateTextButtons);
			var textSelectionButton = new ViewerToolbarButton(f, "TextSelectionButton", "icon icon-WCR_Viewer-TextSelection", false);
			textSelectionButton.setTooltip("viewer", "TextSelectionTT");
			textSelectionButton.listenToTool(this._textSelectionTool, true);
			textSelectionButton.addSubtoolbarComponent(new TextSearchToolbarComponent(f, this._textSearchModel, this._textSelectionModel));
			this._textSelectionButton = textSelectionButton;

			var fontSelectionButton = new ViewerToolbarButton(f, "FontSelectionButton", "icon icon-WCR_Viewer-Font-Recognition", false);
			fontSelectionButton.setTooltip("viewer", "FontSelectionTT");
			fontSelectionButton.listenToTool(this._fontSelectionTool, true);
			this._fontSelectionButton = fontSelectionButton;

			this._highlightTextButton = new CheckFontSizesButton(f, this._checkFontSizesModel, this._highlightFontTool);

			/**
			 * Component: Textselection
			 */
			var textSelectionToolbar = new ToolbarComponent(LayoutConstants.ToolbarComponent.TEXTSELECTION, f);
			this._textSelectionToolbar = textSelectionToolbar;
			/**
			 * Group
			 */
			var selectionGroup = new ButtonGroup("selectionGroup");
			selectionGroup.setTitle(f.i18n, "viewer", "TextBtnGroupTitle");
			textSelectionToolbar.addButtonGroup(selectionGroup);

			// select button
			selectionGroup.addButton(textSelectionButton);
			selectionGroup.addButton(fontSelectionButton);
			selectionGroup.addButton(this._highlightTextButton);

			updateTextButtons();

			this.f.on(
				"module.compare.changed.mode",
				function() {
					setPageForText();
					updateTextButtons();
				}.bind(this)
			);
			this.f.whenModuleAvailable("compare", updateTextButtons);

			return textSelectionToolbar;
		}, this)
	);

	// Toggle the diff layer
	this.f.on(
		"module.compare.changed.diffMode",
		_.bind(function(oldMode, newMode) {
			var diffPage = this.f.getDocumentController().getDifferencePage();
			var activePage = this.f.getActivePage();

			// Do not display difference layer for 'Current Version' and 'Undefined'
			this.f
				.getModule("viewport")
				.toggleLayer(
					"SVGDifferenceLayer",
					diffPage &&
						diffPage.hasSvg() &&
						newMode !== CompareConstants.diffMode.UNDEFINED &&
						newMode !== CompareConstants.diffMode.CURRENTVERSION
				);
			// Do not display current layer for 'Difference Version'
			this.f
				.getModule("viewport")
				.toggleLayer("SVGLayer", activePage && activePage.hasSvg() && newMode !== CompareConstants.diffMode.DIFFVERSION);
		}, this)
	);
};

// inherit
SvgModule.prototype = Object.create(AbstractModule.prototype);
// correct the constructor pointer
SvgModule.prototype.constructor = SvgModule;

/**
 * set the correct pages for the svg layer models
 * @param {AbstractDocumentPage} page
 */
SvgModule.prototype._setSVGLayerModels = function() {
	var page = this.f.getActivePage();
	var hasSVG = page.hasSvg() && !page.hasGraphics();
	var diffPage = this.f.getDocumentController().getDifferencePage();
	var hasDiffSVG = diffPage && diffPage.hasSvg() && !diffPage.hasGraphics();
	if (hasSVG) {
		if (!this._layerModel) {
			this._layerModel = new SvgLayerModel(this.f, "SVGLayer", page, this._textSelectionTool);
		} else {
			this._layerModel.setPage(page);
		}
	}

	if (hasDiffSVG) {
		// could be that diffpage is null -> no problem
		if (!this._diffLayerModel) {
			this._diffLayerModel = new SvgDiffLayerModel(this.f, "SVGDifferenceLayer", diffPage, this._textSelectionTool);
		}
	}

	// toggle svg layers
	this.f.whenViewportAvailable(viewportModule => {
		if (hasSVG) {
			viewportModule.addLayer("SVGLayer", this._layerModel);
			viewportModule.toggleLayer("SVGLayer", true);
		} else {
			viewportModule.toggleLayer("SVGLayer", false);
		}

		if (hasDiffSVG) {
			viewportModule.addLayer("SVGDifferenceLayer", this._diffLayerModel);
			viewportModule.toggleLayer("SVGDifferenceLayer", true);
		} else {
			viewportModule.toggleLayer("SVGDifferenceLayer", false);
		}
	});
};

export default SvgModule;
