import AbstractModel from "@esko/web-viewer/core/AbstractModel";
import rbush from "rbush";
import rbushknn from "rbush-knn";
import _ from "underscore";
import { textFromSelection } from "@esko/web-viewer/svg/TextUtils";
import ShapeUtils from "@esko/shapeutils";
import escapeStringRegexp from "escape-string-regexp";

/**
 * A character used in TextSelectionModel
 */
class Character {
	/**
	 *
	 * @param {Number} minX
	 * @param {Number} minY
	 * @param {Number} maxX
	 * @param {Number} maxY
	 * @param {Number} elementIndex Index of the character. Selection happens from low index to high index
	 * @param {String} character the text character
	 * @param {String} lineIndex all characters that are on the same line should have the same lineIndex. This is used to have nice boxes when rendering selections
	 * @param {String} fontFamily
	 * @param {Number} fontSize
	 */
	constructor(minX, minY, maxX, maxY, elementIndex, character, lineIndex, fontFamily, fontSize) {
		this.minX = minX;
		this.minY = minY;
		this.maxX = maxX;
		this.maxY = maxY;
		this.rect = new ShapeUtils.Rect(new ShapeUtils.Point(minX, minY), new ShapeUtils.Size(maxX - minX, maxY - minY));
		this.elementIndex = elementIndex;
		this.character = character;
		this.lineIndex = lineIndex;
		this.fontFamily = fontFamily;
		this.fontSize = fontSize;
	}
}

/**
 * @exports TextSelectionModel
 * @memberof SvgModule
 * @class The TextSelectionModel is used for representing all text selection tool info
 * @constructor
 * @augments {Core.AbstractModel}
 *
 * @param {Core.ViewerFacade} f - Viewer facade
 * @param {String} privateChannelName - The privateChannelName of the model
 */
var TextSelectionModel = function(f, privateChannelName, page) {
	AbstractModel.call(this, f, privateChannelName);
	this._rbush;
	this._sorted = [];
	this.setPage(page);
	this._boundOnPageChanged = this._onPageChanged.bind(this);
};

TextSelectionModel.prototype = Object.create(AbstractModel.prototype);
TextSelectionModel.prototype.constructor = TextSelectionModel;

/**
 *
 * @param {Array.<Character>} chars
 */
TextSelectionModel.prototype._setCharacters = function(chars) {
	this._rbush = new rbush();
	this._rbush.load(chars);
	this._sorted = chars.sort((x, y) => {
		x.elementIndex - y.elementIndex;
	}); // make sure we have a sorted list
	this._emit("hasText.changed", this.hasText());
};

/**
 *  @param {AbstractDocumentPage} page
 */
TextSelectionModel.prototype.setPage = function(page) {
	if (page === this._page) {
		return;
	}
	if (this._page) {
		this._page.off("property.textData.changed", this._boundOnPageChanged);
	}
	this._page = page;
	if (this._page) {
		this._page.on("property.textData.changed", this._boundOnPageChanged);
	}
	this._onPageChanged();
};

TextSelectionModel.prototype.getPage = function() {
	return this._page;
};

/**
 *
 * @param {Object} point - object with x and y properties
 * @param {Number} [d] - optional distance parameter max distance between point and charachter.
 * @returns {Character} or null if no character found
 */
TextSelectionModel.prototype.getCharacter = function(point, d) {
	return this.getSelection(point, point, d)[0];
};

TextSelectionModel.prototype._transformCharForOutput = function(char) {
	const { minX, maxX, minY, maxY, ...other } = char; // we do not want min
	return other;
};

/**
 * Returns an Array of characters from startPoint to endPoint, distance d is optional
 */
TextSelectionModel.prototype.getSelection = function(startPoint, endPoint, d) {
	if (!this._rbush || !this._sorted || !startPoint || !endPoint) {
		return [];
	}

	var [start] = rbushknn(this._rbush, startPoint.x, startPoint.y, 2, undefined, d);
	var [end] = rbushknn(this._rbush, endPoint.x, endPoint.y, 2);

	if (!start || !end) {
		return [];
	}

	var min = Math.min(start.elementIndex, end.elementIndex);
	var max = Math.max(start.elementIndex, end.elementIndex);
	return _.chain(_.range(min, max + 1))
		.map(i => this._sorted[i])
		.map(char => this._transformCharForOutput(char))
		.compact()
		.value();
};

TextSelectionModel.prototype.getInRect = function(startPoint, endPoint) {
	if (!this._rbush && !startPoint && !endPoint) {
		return [];
	}
	let rect = new ShapeUtils.Rect();
	rect.update(startPoint, endPoint);

	rect.origin.x = startPoint.x;
	rect.origin.y = startPoint.y;

	rect.size.width = endPoint.x - startPoint.x;
	rect.size.height = endPoint.y - startPoint.y;
	rect.normalize();

	const topleft = rect.topleft();
	const bottomright = rect.bottomright();

	return _.chain(this._rbush.search({ minX: topleft.x, minY: topleft.y, maxY: bottomright.y, maxX: bottomright.x }))
		.sortBy(c => c.elementIndex)
		.map(char => this._transformCharForOutput(char))
		.value();
};

/**
 * Is there any text that can be selected
 * @returns {Boolean}
 */
TextSelectionModel.prototype.hasText = function() {
	return this._sorted.length > 0;
};

TextSelectionModel.prototype.getFonts = function(startPoint, endPoint) {
	return _.chain(this.getSelection(startPoint, endPoint))
		.pluck("fontFamily")
		.uniq()
		.value();
};

TextSelectionModel.prototype.getFontSizes = function() {
	return _.chain(this.getCharacters())
		.pluck("fontSize")
		.uniq()
		.map(parseFloat)
		.sort((i, j) => i - j)
		.value();
};

TextSelectionModel.prototype.getCharacters = function() {
	return this._sorted.map(char => this._transformCharForOutput(char));
};

/**
 * Get the text content of the selection
 * @returns {String} - content
 */
TextSelectionModel.prototype.getContent = function(startPoint, endPoint) {
	return this._generateString(this.getSelection(startPoint, endPoint));
};

TextSelectionModel.prototype._onPageChanged = function() {
	if (!this._page || !this._page.get("textData")) {
		this._setCharacters([]);
	} else {
		this._setCharacters(this._page.get("textData"));
	}
};

TextSelectionModel.prototype._generateString = function(selection) {
	return textFromSelection(selection);
};

TextSelectionModel.prototype.findText = function(text, caseSensitive = false) {
	text = text.trim();
	if (!text) {
		return [];
	}

	var charactersWithSpace = _.chain(this._sorted)
		.map((val, i, args) => {
			if (i !== 0 && val.lineIndex !== args[i - 1].lineIndex) {
				return [{ character: " ", fake: true }, val];
			} else {
				return val;
			}
		})
		.flatten()
		.value();
	var str = charactersWithSpace.map(x => x.character).join("");
	var allFounds = [];

	const regExp = new RegExp(escapeStringRegexp(text), caseSensitive ? "g" : "ig");
	let match = regExp.exec(str);
	while (match) {
		allFounds.push(match.index);
		match = regExp.exec(str);
	}

	let textLength = text.length;
	return allFounds.map(x =>
		_.chain(_.range(x, x + textLength))
			.map(i => charactersWithSpace[i])
			.filter(x => x && !x.fake)
			.map(char => this._transformCharForOutput(char))
			.value()
	);
};

export default TextSelectionModel;
export { Character };
