import _ from "underscore";
import AbstractTextTool from "./AbstractTextTool";
import CompareConstants from "@esko/web-viewer/compare/CompareConstants";
import CoreConstants from "@esko/web-viewer/core/CoreConstants";
import ListenerUtil from "@esko/web-viewer/utils/ListenerUtil";
import Mousetrap from "@esko/mousetrap";
import TextSelectionLayerModel from "@esko/web-viewer/svg/TextSelectionLayerModel";
import TextStartEndPointModel from "@esko/web-viewer/svg/TextStartEndPointModel";
import TextSelectionToolSelectionModel from "@esko/web-viewer/svg/TextSelectionToolSelectionModel";

var DEFAULTSTATE = "default";
var DRAGGINGSTATE = "dragging";
var SELECTEDSTATE = "selected";

/**
 * @exports TextSelectionTool
 * @memberof SvgModule
 * @class The TextSelectionTool allows to select text
 * @constructor
 * @augments {AbstractTextTool}
 *
 * @param {Core.ViewerFacade} f - Viewer facade
 * @param {TextSelectionModel}
 * @param {TextSelectionLayerModel}
 */
var TextSelectionTool = function(f, textSelectionModel, textSearchModel) {
	AbstractTextTool.call(this, f, CoreConstants.TOOLS.TEXTSELECTION, textSelectionModel);
	ListenerUtil.mixin(this, f);

	// listen tot popover interaction to change state
	this.currentPopoverListener = null;
	this._textStartEndPointModel = new TextStartEndPointModel(f, this._textSelectionModel);

	/**
	 * @member {TextSelectionLayerModel}
	 * @private
	 */
	this._textSelectionLayerModel = new TextSelectionLayerModel(
		f,
		"text-selection-layer-model",
		textSelectionModel,
		this,
		new TextSelectionToolSelectionModel(f, textSearchModel, this._textStartEndPointModel)
	);

	this.f.whenViewportAvailable(viewportModule => {
		let occurrence = viewportModule.getFirstViewOccurrence("Main");
		if (occurrence && occurrence.getViewContext()) {
			occurrence.getViewContext().addClass("enableTextSelection");
		}
		viewportModule.addLayer("text-selection-layer", this._textSelectionLayerModel);
	});

	f.getDocumentController().whenActiveOrDifferencePageChangedAndParametersAvailableOrChange(_.bind(this._onPageChange, this));

	/**
	 * DEFAULT STATE -> nothing set
	 */
	this.registerState(DEFAULTSTATE, {
		enterState: _.bind(function() {
			this._textStartEndPointModel.setStartPoint(null);
			this._textStartEndPointModel.setEndPoint(null);
		}, this),

		leaveState: _.bind(function() {}, this),

		mousedown: _.bind(function(e) {
			if (e.button === 0) {
				this._textSelectionLayerModel.hidePopover();
				document.activeElement && document.activeElement.blur();
				if (!this.isActive()) {
					// This is only triggered in the SELECTED state (destroys popup and text selection)
					this.changeState(DEFAULTSTATE);
				} else {
					this._textStartEndPointModel.setEndPoint(null);
					var curLoc = this.getTextPoint(e);
					var c = this.getCloseCharacter(curLoc);
					if (c) {
						this._textStartEndPointModel.setStartPoint(curLoc);
						this.changeState(DRAGGINGSTATE);
					}
				}
			}
		}, this),

		mouseup: function() {},

		mousemove: e => {
			var curLoc = this.getTextPoint(e);
			var c = this.getCloseCharacter(curLoc);
			if (c) {
				this.setCursor("cursor-textSelect");
			} else {
				this.setCursor(this.getDefaultCursor());
			}
		},

		mouseout: function() {},

		mouseleave: function() {},

		/**
		 * no context menu!
		 */
		contextmenu: _.bind(function(e) {
			this._textSelectionLayerModel.showPopover(this.getTextPoint(e));
			e.preventDefault();
			e.stopPropagation();
		}, this),

		canBeInterrupted: true
	});

	/**
	 * Dragging STATE -> selecting text
	 */
	this.registerState(
		DRAGGINGSTATE,
		_.defaults(
			{
				enterState: () => {
					this.setCursor("cursor-textSelect");
				},
				mousedown: function() {},

				mouseup: _.bind(function(e) {
					var curLoc = this.getTextPoint(e);
					this._textStartEndPointModel.setEndPoint(curLoc);

					if (curLoc.equals(this._textStartEndPointModel.getStartPoint())) {
						this.changeState(DEFAULTSTATE);
					} else {
						this.changeState(SELECTEDSTATE);
					}
				}, this),

				mousemove: _.bind(function(e) {
					var curLoc = this.getTextPoint(e);
					// update selected text with current location

					if (!curLoc.equals(this._textStartEndPointModel.getStartPoint())) {
						this._textStartEndPointModel.setEndPoint(curLoc);
					}
				}, this)
			},
			this.getState(DEFAULTSTATE)
		)
	);

	/**
	 * selected STATE -> text is selected and annotation balloon is shown
	 */
	this.registerState(
		SELECTEDSTATE,
		_.defaults(
			{
				enterState: _.bind(
					function(e) {
						this._textSelectionLayerModel.showPopover(this._textStartEndPointModel.getEndPoint());
					}.bind(this)
				),

				leaveState: _.bind(function() {
					this._textSelectionLayerModel.hidePopover();
					if (this.currentPopoverListener) {
						this._unregisterListener(this.currentPopoverListener);
					}
				}, this)
			},
			this.getState(DEFAULTSTATE)
		)
	);

	this.registerActiveEvents(["mousedown", "mousemove", "mouseup", "mouseleave", "contextmenu"]);

	// Esc allows to reset selection
	this.activeKeyEvents.push([
		"esc",
		_.bind(function(e) {
			this.changeState(DEFAULTSTATE);
			e.preventDefault();
			e.stopPropagation();
		}, this),
		"keyup"
	]);

	/**
	 * @override
	 */
	this._override("initialize", function(widget, toolManager) {
		this._super(widget, toolManager);
		this.changeState(DEFAULTSTATE);
	});

	/**
	 * @override
	 */
	this._override("activate", function(interruptRequest) {
		this._super(interruptRequest);
		if (this.f.getModule("viewport")) {
			this.f
				.getModule("viewport")
				.getFirstViewOccurrence("Main")
				.getViewContext()
				.addClass("enableTextSelection");
		}
		this.f.getModule("viewport").toggleLayer("text-selection-layer", true);
	});

	/**
	 * @override
	 */
	this._override("deactivate", function(interruptRequest) {
		this._super(interruptRequest);
		if (!interruptRequest) {
			this.changeState(DEFAULTSTATE);
			this.f.getModule("viewport").toggleLayer("text-selection-layer", false);
			if (this.f.getModule("viewport")) {
				this.f
					.getModule("viewport")
					.getFirstViewOccurrence("Main")
					.getViewContext()
					.removeClass("enableTextSelection");
			}
		}
	});

	// activate text selection tool
	this.passiveKeyEvents.push([
		"t",
		_.bind(function(e) {
			if (this._textSelectionModel.hasText()) {
				e.preventDefault();
				f.emit("toolManager.activateTool", this);
			}
		}, this),
		"keypress"
	]);

	Mousetrap.bind(
		"ctrl+a",
		e => {
			if (textSelectionModel.hasText()) {
				f.emit("toolManager.activateTool", CoreConstants.TOOLS.TEXTSELECTION);
				textSearchModel.selectAll();
				e.preventDefault();
			}
		},
		"keypress"
	);
};

// extend from the AbstractTextTool
// inherit
TextSelectionTool.prototype = Object.create(AbstractTextTool.prototype);
TextSelectionTool.prototype.constructor = TextSelectionTool;

TextSelectionTool.prototype._onPageChange = function() {
	if (this.isActive()) {
		this.changeState(DEFAULTSTATE);
	}
};

TextSelectionTool.prototype.canBeInterrupted = function(tool) {
	return false;
};

/**
 * Determines if this tool can be activated
 *
 * @returns  {Boolean}
 */
TextSelectionTool.prototype.canBeActivated = function() {
	var compareModule = this.f.getModule("compare");
	if (compareModule) {
		var mode = compareModule.getActiveMode();
		var allowedModes = [CompareConstants.mode.SHOW_CURRENT, CompareConstants.mode.SHOW_DIFFERENCE];
		return !mode || allowedModes.indexOf(mode.getName()) !== -1;
	}
	return true;
};

export default TextSelectionTool;
