/**
 * @namespace WebviewerModule
 */
/**
 * @namespace WebviewerModule
 */
import _ from "underscore";

import AbstractModule from "@esko/web-viewer/core/AbstractModule";
import WebviewerWidgetModel from "@esko/web-viewer/webviewer/WebviewerWidgetModel";
import ListenerUtil from "@esko/web-viewer/utils/ListenerUtil";

/**
 * @exports WebviewerModule
 * @memberof WebviewerModule
 * @class
 * The WebviewerModule, this module installs the correct models for the viewport and load Graphics or Cad module
 * @constructor
 * @augments Core.AbstractModule
 * @param {Core.ViewerFacade} f
 * @param {Boolean} silenced
 */
var WebviewerModule = function(f, silenced) {
	AbstractModule.call(this, f, silenced, "webviewer");
	ListenerUtil.mixin(this, f);

	this.webviewerWidgetModel = new WebviewerWidgetModel(f);

	var docController = this.f.getDocumentController();
	this._pageChanged(undefined, this.f.getActivePage());
	this._pageChanged(undefined, docController.getCadOverlay());

	if (f.getModule("viewport")) {
		f.getModule("viewport").setWidgetModel(this.webviewerWidgetModel);
	} else {
		this.f.whenModuleAvailable(
			"viewport",
			module => {
				this._pageChanged(undefined, this.f.getActivePage());
				this._pageChanged(undefined, docController.getCadOverlay());
				module.setWidgetModel(this.webviewerWidgetModel);
			},
			undefined,
			this
		);
	}

	// react when the active page has changed
	docController.whenActivePageChanged(this._pageChanged, this, this);
	docController.whenCadOverlayChanged(this._pageChanged, this, this);
	docController.whenDifferencePageChanged(this._pageChanged, this, this);
};

// inherit
WebviewerModule.prototype = Object.create(AbstractModule.prototype);
// correct the constructor pointer
WebviewerModule.prototype.constructor = WebviewerModule;

WebviewerModule.prototype.destroy = function() {
	this._unregisterAllListeners();
};

/**
 * The active page or CAD overlay has changed, load the needed modules, and create a new view
 *
 * @param {Data.AbstractDocumentPage} oldPage
 * @param {Data.AbstractDocumentPage} newPage
 */
WebviewerModule.prototype._pageChanged = function(oldPage, newPage) {
	var mainView = this.webviewerWidgetModel.getViewModel("Main");

	if (mainView === undefined || mainView === null) {
		this.webviewerWidgetModel.createView("Main", "", true);
	}

	if (newPage) {
		// create main view
		if (newPage.hasCad() || newPage.hasLinkedPage()) {
			this.f.requestModule("cad", undefined, this.silenced);
		}

		if (newPage.hasSvg()) {
			this.f.requestModule("svg", undefined, this.silenced);
		}
		if (newPage.hasVideo()) {
			this.f.requestModule("video", undefined, this.silenced);
		}
		if (newPage.hasGraphics() || newPage.isBlankPage()) {
			this.f.requestModule("graphics", undefined, this.silenced);
		}

		// load 3D module if there is a 3D perspective or a 3d perspective can be generated
		if (newPage.has3D() || this.f.getProperty("show3D")) {
			this.f.requestModule("view3D", undefined, this.silenced);
			// Show 3D Model immediately if 'show3D' config is set to true
		}

		newPage.whenParametersAvailable(
			function() {
				if ((newPage.hasGraphics() || newPage.isBlankPage() || newPage.hasSvg()) && newPage.getDocumentVersion().isMultiPage()) {
					this.f.requestModule("pagelist", undefined, this.silenced);
				}
			},
			this,
			this
		);
	}
};

/**
 *	Overrride the setSilenced method to also set the SVG Module as non silenced, indicating that it can publish its tools etc.
 * 	@override
 */
WebviewerModule.prototype.setSilenced = function(isSilenced) {
	AbstractModule.prototype.setSilenced.call(this, isSilenced);
	if (!isSilenced) {
		this._getManagedModules().forEach(
			function(mod) {
				this.f.whenModuleAvailable(mod, function(mod) {
					mod.setSilenced(false);
				});
			}.bind(this)
		);
	}
};

/**
 * Generates a list of modules that can be silenced by this module
 *
 * @returns {Array<String>}
 */
WebviewerModule.prototype._getManagedModules = function() {
	return ["svg", "cad", "graphics", "view3D", "pagelist", "video"];
};

export default WebviewerModule;
